#!/usr/bin/env python3

__version__ = '1.0.0'
__website__ = 'https://www.fb.gv.at/Funk/amateurfunkdienst.html'

import argparse
import os
import mariadb
import sys
import time
import pypdf
from pypdf import PdfReader
import re # regular expression
import datetime
import configparser

from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.chrome.service import Service as ChromiumService

def call_description():
	print(f'Download and Parse the Austrian Callbook Version {__version__}')

def call_configparser():
	config = configparser.ConfigParser()
	try:
		configfile = get_active_path('config.ini')
		ret = config.read(configfile)
		if not ret:
			print(f'The file {configfile} was not found. We create a new config file.')
			config['Common']={
				'description': 'Download and Parse the Austrian Callbook',
				'author': 'Thomas Kuschel, OE3TKT (OExTKT)',
				'version': __version__,
				'created': datetime.datetime.now()
			}
			config['MariaDB'] = {
				'user':		'om',
				'password':	'oe3tkt',
				'host':		'127.0.0.1',
				'port':		3306,
				'database':	'callbook'}
			with open(get_active_path('config.ini'), 'w') as configfile:
				config.write(configfile, False)
				print(f'Configuration file "config.ini" created.')

	except FileNotFoundError:
		sys.exit(8)
	except Exception as e:
		print('Error: {}'.format(e), file=sys.stderr)
		sys.exit(9)
	# print(config.sections())   ## ['Common', 'MariaDB']
	return config

def call_parser():
	parser = argparse.ArgumentParser(
		description='Download and Parse the Austrian Callbook',
		epilog=f'''
		Written by Thomas Kuschel,
		Version {__version__}
		'''
	)
	parser.add_argument('-i', '--interactive', action='store_true', default=False)
	# parser.add_argument('-s', '--server' default=__website__, required=False)
	parser.add_argument('-V', '--version', action='version', version='{} {}'.format(os.path.split(__file__)[1],__version__))
	parser.add_argument('-v', '--verbose', action='append_const', const = 1)
	# Rufzeichenliste_AT_Stand_010624.pdf
	parser.add_argument('-p', '--path', default='', help= 'skip the download if the specified path to a PDF file exists')
	# parser.add_argument('-t', '--type', default='' , help='specify the output, supported types are [ CSV | JSON ]') # not implemented yet
	parser.add_argument('-o', '--output', default='', help='specify the file where the data are written to, default stdout')
	parser.add_argument('-m', '--mariadb', help='SQL interface to MariaDB (MySql) format "<IP-Address>:<Port> <User> <Passwd>" or defined in .config')
	parser.add_argument('url', metavar='URL', nargs='?', default=__website__)

	opt = parser.parse_args()
	opt.verbose = 0 if opt.verbose is None else sum(opt.verbose)
	if opt.path != '' and os.path.dirname(opt.path) == '':
		opt.path = get_active_path(opt.path)
	return opt

def call_website(url,verbose,path='',interactive=False,output='',mariadb=''):
	if path:
		if os.path.exists(path):
			return path
		else:
			print(f'The given path "{path}" does not exist.')
			sys.exit(3)
	if(interactive):
		print('Interactive')
		driver=webdriver.Chrome()
	else:
		print('Headless Script')
		options = webdriver.ChromeOptions()
		options.add_argument('--headless')
		options.add_argument('--no-sandbox')
		options.add_argument('--disable-dev-shm-usage')
		driver = webdriver.Chrome(options=options)

	driver.get(url)
	print(driver.title)
	# elements = driver.find_elements(By.XPATH,'//a[contains(@href,"Rufzeichen")]')
	elements = driver.find_elements(By.PARTIAL_LINK_TEXT,"Rufzeichen")

	if elements:
		element = elements[0]
		href = element.get_attribute('href')
		filename = element.click() # take the first one
	else:
		print('Sorry, no Link containing "Rufzeichen" found.')
		driver.close()
		sys.exit(2)

	print(element.text)
	# print(href)
	if(interactive):
		time.sleep(300)
	else:
		time.sleep(4)

	driver.close()
	return os.path.basename(href)

def remove_first_quote_if_odd(text, verbose = 0):
	double_quote_cnt = text.count('"')
	# single_quote_cnt = text.count("'")

	if (double_quote_cnt % 2 != 0): # or (single_quote_cnt % 2 != 0):
		# Find and remove the first quote
		for i, char in enumerate(text):
				if char in ['"']: # ['"', "'"]:
					if (verbose > 1):
						print(text)
					text = text[:i] + text[i+1:]
					if (verbose > 1):
						print(text)
					break
	return text

def is_clubstation(call):
	assert(len(call) > 3)
	if call[3].upper() == 'X' or call.upper() == 'OE5SIX': # special case with OE5SIX (Clubstation)
		return True

	return False

def replace_substring_with_line(path, search_substring, verbose=0):

	try:
		if not replace_substring_with_line.lines:
			with open(path, 'r') as file:
				replace_substring_with_line.lines = file.readlines()

		for line in replace_substring_with_line.lines:
			if search_substring[0:46].lower() in line.lower():
				return line.strip()

	except FileNotFoundError:
		print(f'The file {path} was not found.')
	except Exception as e:
		print(f'An error occurred: {e}')

	return search_substring

replace_substring_with_line.lines = None

def gender_substring(path, search_substring, verbose=0):
	try:
		if not gender_substring.lines:
			with open(path, 'r') as file:
				gender_substring.lines = file.readlines()

		for line in gender_substring.lines:
			if line[2:].strip() == search_substring: # search from position 2 and remove all spaces or \n chars
				return line[0] # return the char of gender i.e. 'f' or 'm'
	except FileNotFoundError:
		print(f'The file {path} was not found.')
	except Exception as e:
		print(f'An error occurred: {e}')

	return 'x' # not found, unknown gender

gender_substring.lines = None

def get_active_path(file):
	return os.path.join(os.path.dirname(os.path.abspath(__file__)), file)

def get_gender(firstnames, surname, call, verbose=0):

	# load the .gender file:
	genderfile = '.gender'
	gender = 'x'
	gpath = get_active_path(genderfile)
	if os.path.exists(gpath):
		firstname = firstnames.split(' ', 1)[0]
		gender = gender_substring(gpath, firstname, verbose=0) # only check 1st/firstname of name, important when there are more than 1 firstnames 
		if gender == 'x':
			if verbose > 0:
				get_gender.cnt += 1
				print(f'({get_gender.cnt}){call} "{firstname}" [{firstnames} {surname}] not found in file {genderfile} - gender "x" is set.')

	return gender

get_gender.cnt = 0

def call_split_name(fullname, call, verbose):

	assert(len(fullname) > 1)

	name = fullname.split(' ', 1)
	surname = name[0]
	# several special cases like surname "de Lijezer", "van Dijk", "el Shamaa", "da Silva", etc.
	match surname.lower():
		case 'de' | 'el' | 'da':
			name = fullname[3:].split(' ',1)
			surname = surname.lower() + ' ' + name[0]
			if verbose > 1:
				print(f'## {fullname} --> {surname} ##')

		case 'van' | 'von' :
			name = fullname[4:].split(' ',1)
			surname = surname.lower() + ' ' + name[0]
			if surname.lower() in ['van der', 'von der', 'van den']:	# e.g. "van der Meulen", "Walther von der Vogelweide", "Annie van den Berg"
				name = fullname[8:].split(' ',1)
				surname = surname.lower() + ' ' + name[0]
			if verbose > 1:
				print(f'## {fullname} --> {surname} ##')
		case 'della' : # Ancient Italian noble family "della Rowere"
			name = fullname[6:].split(' ',1)
			surname = surname.lower() + ' ' + name[0]
			if verbose > 1:
				print(f'## {fullname} --> {surname} ##')
		case 'senarclens' : # Senarclens de Grancy
			if fullname.lower().startswith('senarclens de grancy'):
				name = fullname[21:].split(' ',1)
				surname = 'Senarclens de Grancy'
				if verbose > 1:
					print(f'## {fullname} --> {surname} ##')

	if len(name) > 1:
		firstname = name[1].lstrip() # FIX when there are more than 1 space b/w surname and firstname
	else:
		firstname = '<unknown>'

	# In Austria the call suffix starting with Y is an YL (young lady)
#	if call[3].upper() == 'Y':
	if False:
		gender = 'f'
	else:
		gender = get_gender(firstname, surname, call, verbose)

	return firstname, surname, gender

def fix_call(call,verbose=1):
	fixfile = '.typo_call'
	path = get_active_path(fixfile)
	if os.path.exists(path):
		if not fix_call.lines:
			with open(path,'r') as file:
				fix_call.lines = file.readlines()
				if verbose > 0:
					print(f'File "{fixfile}":')
					for line in fix_call.lines:
						print(f'>> {line.rstrip()}')
					print('>> ** EOF **')
		for line in fix_call.lines[1:]: # starting with line 1
			words = line.rstrip().split()
			if len(words) != 2:
				continue
			if words[0] == call:
				if verbose > 0:
					print(f'Corr typo call: {words[0]} => {words[1]}')
				return words[1]

	return call
fix_call.lines = None

def fix_typo(call, fullname, verbose=1):
	fixtypofile = '.typo_callbook'
	path = get_active_path(fixtypofile)
	if os.path.exists(path):
		try:
			if not fix_typo.lines:
				with open(path, 'r') as file:
					fix_typo.lines = file.readlines()
					if verbose > 0:
						print(f'File "{fixtypofile}":')
						for line in fix_typo.lines:
							print(f'>> {line.rstrip()}')
						print('>> ** EOF **')

			for line in fix_typo.lines[4:]: # starting with line 4
				if len(fix_typo.spaces) == 0: # not initialized
					if line[0] == '*':
						words = line.split()
						assert len(words) == 4 # i.e. '*, call, nachname, vorname'
						fix_typo.spaces = [line.index(words[1]), line.index(words[2]), line.index(words[3])]
				else:
					if call in line[2:8]:
						if verbose > 1:
							print(f'Call: {call} found')
						match line[0]:
							case '#':
								if verbose > 1:
									print(line.rstrip())
							case 'F' | 'N':
								if verbose > 0:
									print(line.rstrip())
									print(fullname)
								firstname1, surname1, gender1 = call_split_name(fullname, call, 0)
								fullname2 = line[fix_typo.spaces[1]:fix_typo.spaces[2]-1].rstrip() + ' ' + line[fix_typo.spaces[2]:-1]
								if verbose > 0:
									print(fullname2)
								firstname2, surname2, gender2 = call_split_name(fullname2, call, 0)
								# Hardening: at a minimum, either the firstnames or the surenames must fit
								fix_cnt = 0
								if (firstname1 != firstname2):
									fix_cnt += 1
								if (surname1 != surname2):
									fix_cnt += 1
								# when the surname is splitted and wrong written:
								if line[0] == 'N' and firstname2 in fullname:
									if fix_cnt > 1: fix_cnt = 1
								if fix_cnt == 0:
									print(f'It is fixed! You can remove the line with the item {call} from the file {fixtypofile}!')
								elif fix_cnt > 1:
									print(f'Something went wrong, there are several bugs. Check line with call {call} in file {fixtypofile}!')
								else:
									fullname = fullname2
							case 'X': # exchange the surname with firstname
								if verbose > 0:
									print(line.rstrip())
									print(fullname)
								firstname1, surname1, gender1 = call_split_name(fullname, call, 0)
								fullname2 = line[fix_typo.spaces[1]:fix_typo.spaces[2]-1].rstrip() + ' ' + line[fix_typo.spaces[2]:-1]
								if verbose > 0:
									print(fullname2)
								firstname2, surname2, gender2 = call_split_name(fullname2, call, 0)
								fix_cnt = 0
								if (firstname1 ==  firstname2) and (surname1 == surname2):
									print(f'It is fixed! You can remove the line with the item {call} from the file {fixtypofile}!')
								elif (firstname1 !=  surname2) or (surname1 != firstname2):
									print(f'Something went wrong, there are several bugs. Check line with call {call} in file {fixtypofile}!')
								else:
									fullname = fullname2

		except FileNotFoundError:
			print(f'The file {path} was not found.')
		except Exception as e:
			print(f'An error occurred: {e}')

	return fullname
fix_typo.lines = None
fix_typo.spaces = []

def isinteger(s):
	for char in s:
		if not char.isdigit():
			return False
	return True

def call_change_city(location, address, verbose):
	post_file = '.post_code'
	path = get_active_path(post_file)
	if not call_change_city.lines:
		if os.path.exists(path):
			try:
				with open(path, 'r') as file:
					call_change_city.lines = file.readlines()
					if verbose > 0:
						print(f'File "{post_file}":')
						for line in call_change_city.lines:
							print(f'>> {line.rstrip()}')
						print('>> ** EOF **')
				# print(f'{path}')
			except FileNotFoundError:
				print(f'The file {path} was not found.')
			except Exception as e:
				print(f'An error occurred: {e}')
		else:
			return 0,'',''

	postal_code = 0
	city = location
	addr,city = corr_street(address,city)
	street = addr

	for line in call_change_city.lines:
		line = line.rstrip() # remove the \n
		locpart = line.split(' ',1)
		if len(locpart) < 2:
			continue
		exchange = locpart[1].split('#')
		citypart = exchange[0].split('|',1)
		if (citypart[0] != location): # A hit? Else... 
			continue
		if len(citypart) > 1:
			# addr = address[0:len(citypart[1])]
			if (citypart[1] != addr[0:len(citypart[1])]):
				continue
		postal_code = locpart[0]
		if len(exchange) > 1:
			city = exchange[1]
			if len(exchange) > 2 and len(citypart) > 1:
				street = addr.replace(citypart[1],exchange[2])
		# print(f'PLZ: {postal_code}, City: {city}')
		break
	if postal_code != 0:
		if verbose > 0:
			if address == addr:
				print(f'location/address: {location}|{addr} => {postal_code} {city}|{street}')
			else:
				print(f'location/address: {location}|({address}){addr} => {postal_code} {city}|{street}')
	else:
			call_change_city.cnt += 1
			if address == addr:
				print(f'[{call_change_city.cnt}] loc/addr: {location}|{addr} => {postal_code} {city}|{street}')
			else:
				print(f'[{call_change_city.cnt}] loc/addr: {location}|({address}){addr} => {postal_code} {city}|{street}')

	return postal_code, city, street

call_change_city.lines = None
call_change_city.cnt = 0

def corr_street(street,city):
	street = street.replace('strasse','straße')
	street = street.replace('Strasse','Straße')
	street = street.replace('str ','straße ')
	street = street.replace('Str ','Straße ') # Brünner Str 221/4/5
	street = street.replace('str.','straße')
	street = street.replace('Stg. ','Stiege ')
	street = street.replace('g. ','gasse ')
	street = street.replace('-G ','-Gasse ')
	street = street.replace('Stiege ','Stg. ')
	street = street.replace('Dr. ','Dr.-')
# Special, stupid fixes
	street = street.replace('Str.1','Straße 1') # Peter-Berner-Str.10/1/10
	street = street.replace('Str.5','Straße 5') # Wagramer Str.59/14/2
	street = street.replace('Str.6','Straße 6')
	street = street.replace('Str.8','Straße 8')
	street = street.replace('Str.','Straße') # Brünner Str. 106/43
	street = street.replace('Str7','Straße 7') # Hetzendorfer Str79/2/14
	street = street.replace('Str8','Straße 8') # Theodor-Kramer-Str8/274
	street = street.replace('Markhofg.13-15/','Markhofgasse 13-15/5/14') # Mautner-Markhofg.13-15/ (OE1JJB)
	street = street.replace('g.2','gasse 2') # Spittelbreiteng.23/7/2/
	street = street.replace('g.3','gasse 3') # Rathausg.32, Wulkaprodersdorf
	street = street.replace('g.4','gasse 4') # Siebenbrunneng.42/1/18
	street = street.replace('Absbergg ','Absberggasse ')
	street = street.replace('Alfred Huthg ','Alfred-Huth-Gasse ')
	street = street.replace('Altmannsdfstraße ','Altmannsdorfer Straße ') # Altmannsdfstr 170 23/7
	street = street.replace('Amthofensiedlung1','Amthofensiedlung 1')
	street = street.replace('Anningerg ','Anningergasse ')
	street = street.replace('Antonig ','Antonigasse ') # 7063 Oggau am Neusiedler See
	street = street.replace('Arbeiterg ','Arbeitergasse ')
	street = street.replace('Arnoldg ','Arnoldgasse ')
	street = street.replace('Bellinig ','Bellinigasse ')
	street = street.replace('Birkeng ','Birkengasse ')
	street = street.replace('Bucheng ','Buchengasse ')
	street = street.replace('Canisiusg ','Canisiusgasse ')
	street = street.replace('Czartoryskig ','Czartoryskigasse ')
	street = street.replace('Denglerg ','Denglergasse ')
	street = street.replace('Endemanng ','Endemanngasse ')
	street = street.replace('Ernst Ludwigg ','Ernst-Ludwig-Gasse ')
	street = street.replace('Feldg ','Feldgasse ')
	street = street.replace('Flachg ','Flachgasse ')
	street = street.replace('Gatterburgg ','Gatterburggasse ')
	street = street.replace('Georg Sigl G ','Georg-Sigl-Gasse ')
	street = street.replace('Goldegg ','Goldeggasse ')
	street = street.replace('Gregoryg ','Gregorygasse ')
	street = street.replace('Grundlg ','Grundlgasse ')
	street = street.replace('Gusenleithnerg ','Gusenleithnergasse ')
	street = street.replace('Sendemast Hegerber','Sendemast Hegerberg')
	street = street.replace('Hermanng ','Hermanngasse ') # 7, Neubau, Wien
	street = street.replace('Hutteng ','Huttengasse ')
	street = street.replace('Isbaryg ','Isbarygasse ')
	street = street.replace('Johannesg ','Johannesgasse ')
	street = street.replace('Karl Stummerg ','Karl-Stummer-Gasse ') # Wieselburg
	street = street.replace('Kaunitzg ','Kaunitzgasse ')
	street = street.replace('Keferg ','Kefergasse ')
	street = street.replace('Keinerg ','Keinergasse ')
	street = street.replace('Khekg ','Khekgasse ')
	street = street.replace('Lacknerg ','Lacknergasse ')
	street = street.replace('Langsulzg ','Langsulzgasse ')
	street = street.replace('Lavantg ','Lavantgasse')
	street = street.replace('Leiteng ','Leitengasse ')
	street = street.replace('Leopoldig ','Leopoldigasse ')
	street = street.replace('Lichtensterng ','Lichtensterngasse ')
	street = street.replace('Linseng ','Linsengasse ')
	street = street.replace('Markowskyg ','Markowskygasse ')
	street = street.replace('Mölkerg ','Mölkergasse ')
	street = street.replace('Mommseng ','Mommsengasse ')
	street = street.replace('Neilreichg ','Neilreichgasse ')
	street = street.replace(' Ors Sendeanlage',' ORS-Sendeanlage')
	street = street.replace('Orf Gebäude','ORF-Gebäude')
	street = street.replace('Othellog ','Othellogasse ')
	street = street.replace('Pehamg ','Pehamgasse ')
	street = street.replace('Pfeilg ','Pfeilgasse ')
	street = street.replace('Pfenninggeldg ','Pfenninggeldgasse ')
	street = street.replace('Pohlg ','Pohlgasse ')
	street = street.replace('Reinprechtsdf ','Reinprechtsdorfer ')
	street = street.replace('Ringelseeg ','Ringelseegasse ')
	street = street.replace('Robert-Hamerlingg ','Robert-Hamerling-Gasse ')
	street = street.replace('Romanog ','Romanogasse ')
	street = street.replace('Rombergg ','Romberggasse ')
	street = street.replace('Roseng ','Rosengasse ')
	street = street.replace('Rötzerg ','Rötzergasse ')
	street = street.replace('Ruzickag ','Ruzickagasse ')
	street = street.replace('Schützpl ','Schützplatz ')
	street = street.replace('Schiffmühlenst ','Schiffmühlenstraße ')
	street = street.replace('Schirrmanng ','Schirrmanngasse ')
	street = street.replace('Schlussnig ','Schlußnig ')
	street = street.replace('Seutterg ','Seuttergasse ')
	street = street.replace('Siedlerg ','Siedlergasse ')
	street = street.replace('Sigmund-Haffnerg ','Sigmund-Haffner-Gasse ')
	street = street.replace('Sillerg ','Sillergasse ')
	street = street.replace('Speckbacherg ','Speckbachergasse ')
	street = street.replace('Spengerg ','Spengergasse ')
	street = street.replace('Starhembergg ','Starhemberggasse ')
	street = street.replace('Stolbergg ','Stolberggasse ')
	street = street.replace('Straßmeyerg ','Straßmeyergasse ')
	street = street.replace('Zanaschkag ','Zanaschkagasse ')
	street = street.replace('Zentag ','Zentagasse ')
	street = street.replace('aße1','aße 1')
	street = street.replace('aße2','aße 2') #Ziegelhofstrasse29/16
	street = street.replace('aße3','aße 3')
	street = street.replace('aße5','aße 5') #Josefstädterstraße56/2
	street = street.replace('aße6','aße 6')
	street = street.replace('aße8','aße 8')
	street = street.replace('aße9','aße 9')
	street = street.replace('dorf3','dorf 3') # Füllersdorf 37
	street = street.replace('An Der Lan Straße','An-der-Lan-Straße')
	street = street.replace('An-Der-','An-der-') # An-der-Lan-Straße
	street = street.replace('An Der Furt','An-der-Furt')
	street = street.replace('Auf Der ','Auf der ')
	#street = street.replace('Auf Der Kohlwiese','Auf der Kohlwiese') # Walding
	street = street.replace(' (Klg Am Am', ' (Klg Am Ameisbach)')
# Typos
	street = street.replace('Achtungvierzigerplatz ','Achtundvierzigerplatz ')
	street = street.replace('Aflinger Straße','Aflingerstraße')
	street = street.replace('Anton-Bruckner- Straße','Anton-Bruckner-Straße')
	street = street.replace('Arsenal Objet ','Arsenal Objekt ')
	street = street.replace('Basler G ','Basler Gasse ')
	street = street.replace('Bierhaeuselberggasse ','Bierhäuselberggasse ')
	street = street.replace('Breitenfrt ','Breitenfurter ')
	street = street.replace('Brigitt.-L148-154/12/17','Brigittenauer Lände 148-154/12/17')
	street = street.replace('Brigitt L ','Brigittenauer Lände ')
	street = street.replace('Dr.Sumpfstraße ','Dr.-Stumpf-Straße ')
	street = street.replace('Fesstgasse ','Feßtgasse ')
	street = street.replace('Fuchsenfeld Gasse ','Fuchsenfeldgasse')
	street = street.replace('Gußriegelstraße','Gussriegelstraße') # seit 1999-12-17 Gemeinderatsbeschluss Wien
	street = street.replace('Hardegggasse','Hardeggasse')
	street = street.replace('Hoehenrainweg ','Höhenrainweg ') # Ehrwald
	street = street.replace('Hoeggen ','Höggen ')
	street = street.replace('Hoergas ','Hörgas ')
	street = street.replace('H.-Collin-Straße','Heinrich-Collin-Straße')
	street = street.replace('Hptst ','Hauptstraße ')
	street = street.replace('Hernalser Hptstraße ','Hernalser Hauptstraße ') # Hernalser Hptstr.230/14
	street = street.replace('Hütteldfstr1','Hütteldorfer Straße 1') # Hütteldfstr180-186/5/29
	street = street.replace('In Der ','In der ')
	street = street.replace('Jaegerstraße ','Jägerstraße ')
	street = street.replace('Kaiserebersdorferst ','Kaiserebersdorfer Straße ')
	street = street.replace('Kaiser Ebersd.Straße','Kaiserebersdorfer Straße')
	street = street.replace('Kuernbergblick','Kürnbergblick')
	street = street.replace('Linzerstraße ','Linzer Straße ')
	street = street.replace('Loewenthalgasse ','Löwenthalgasse ')
	street = street.replace('Loeblichgasse','Löblichgasse')
	if (city=='Graz'):
		street = street.replace('Maderspergerstraße','Maderspergergasse')
	street = street.replace('Neulerchfd.Straße','Neulerchenfelder Straße')
	street = street.replace('Neustift Am Walde','Neustift am Walde')
	street = street.replace('Nussdorfer','Nußdorfer')
	street = street.replace('Oberweissenbach','Oberweißenbach')
	street = street.replace('Oehlermuehle ','Öhlermühle ')
	street = street.replace('Pernersdorfergasse','Pernerstorfergasse') # Wien 10
	street = street.replace('Puechlgasse','Püchlgasse')
	street = street.replace('Rossauer Lände','Roßauer Lände')
	street = street.replace('Rossbachstraße','Roßbachstraße')
	street = street.replace('Russbergstraße','Rußbergstraße')
	street = street.replace('Salzlaende ','Salzlände ')
	street = street.replace('Schliessmanngasse ','Schließmanngasse ')
	street = street.replace('Schlossmuehlgasse ','Schloßmühlgasse ') # Wiener Neudorf
	street = street.replace('Schluesselamtsgasse ','Schlüsselamtsgasse ') # Krems
	if (city == 'Innsbruck'):
		street = street.replace('Schmidgasse','Schmiedgasse')
	street = street.replace('Schuchardstraße','Schuchardtstraße')
	street = street.replace('Schuetzenstraße','Schützenstraße')
	street = street.replace('Simmering/Blockkraftw','Simmering Blockkraftwerk')
	street = street.replace('Spesingerstraße ','Speisingerstraße') # typo in Wien
	street = street.replace('St. Hubertusgasse ','St.-Hubertus-Gasse ')
	street = street.replace('St.Michael-G.1','St.-Michael-Gasse 1')
	street = street.replace('St. Veit-Gasse','St.-Veit-Gasse') # 1130 Wien
	street = street.replace('Stoegergasse','Stögergasse') # 2000 Stockerau
	street = street.replace('Sturbergasse','Strubergasse') # Sturbergasse 60, Salzburg
	street = street.replace('Toescherweg','Töscherweg') # 8144 Tobelbad
	street = street.replace('Universitaetsstraße','Universitätsstraße') # 6020 Innsbruck
	street = street.replace('Voegelebichl','Vögelebichl') # 6020 Innsbruck
	street = street.replace('Waelderstraße','Wälderstraße') # 6900 Bregenz
	street = street.replace('Weissenbachgraben ','Weißenbachgraben ') # Waidhofen an der Ybbs
	if (street == 'Rum,Serlesstraße 22'):
		city = 'Rum'
		street = 'Serlesstraße 22'
# Spelling VIPs
	street = street.replace('10. Oktober Straße','10.-Oktober-Straße')
	street = street.replace('12.Februarstraße','12.-Februar-Straße')
	street = street.replace('12. Novemberstraße','12.-November-Straße')
	street = street.replace('Ada Christengasse','Ada-Christen-Gasse')
	street = street.replace('Adalbert Stifter Straße','Adalbert-Stifter-Straße')
	street = street.replace('Adolf Netsch Straße','Adolf-Netsch-Straße')
	street = street.replace('Alexander Groß-Gasse','Alexander-Groß-Gasse')
	street = street.replace('Alfons Petzoldgasse','Alfons-Petzold-Gasse')
	street = street.replace('Alfred Jägerweg','Alfred-Jäger-Weg')
	street = street.replace('Alois Kellner Straße','Alois-Keller-Straße')
	street = street.replace('Am Langen Felde','Am langen Felde')
	street = street.replace('Am Muellerbach','Am Müllerbach')
	street = street.replace('Andreas Grein Gasse','Andreas-Grein-Gasse')
	street = street.replace('Andreas Hoferstraße','Andreas-Hofer-Straße') # Landeck
	street = street.replace('Andreas Urteil Weg','Andreas-Urteil-Weg')
	street = street.replace('Anton Baumgartner Straße','Anton-Baumgartner-Straße')
	street = street.replace('Anton Baumgartnerstraße','Anton-Baumgartner-Straße')
	street = street.replace('Anton-Baumgartnerstraße','Anton-Baumgartner-Straße')
	street = street.replace('A Baumgartnerstraße','Anton-Baumgartner-Straße')
	street = street.replace('A. Baumgartnerstraße','Anton-Baumgartner-Straße')
	street = street.replace('Anton Benya Straße','Anton-Benya-Straße')
	street = street.replace('Anton Böck Gasse','Anton-Böck-Gasse')
	street = street.replace('Anton Brucknergasse','Anton-Bruckner-Gasse')
	street = street.replace('Anton Bruckner-Straße','Anton-Bruckner-Straße') # 6300 Wörgl
	street = street.replace('Anton-Ehrenfriedstraße','Anton-Ehrenfried-Straße') # 2020 Hollabrunn
	street = street.replace('Anton Haglg','Anton-Hagl-Gasse') # Gablitz
	street = street.replace('Anton-Hobbinger-Gasse','Anton-Hobbiger-Gasse') # Heidenreichstein
	street = street.replace('Anton Keller Weg','Anton-Keller-Weg')
	street = street.replace('Anton Kriegergasse','Anton-Krieger-Gasse')
	street = street.replace('Anton Mühlbacherstraße','Anton-Mühlbacher-Straße')
	street = street.replace('Anton Regner-Straße','Anton-Regner-Straße')
	street = street.replace('Anton Ullrich Gasse','Anton-Ullrich-Gasse') # Heidenreichstein
	street = street.replace('A-Weiser Straße','Anton-Weiser-Straße')
	street = street.replace('A.W. Pragergasse','Anton-Wenzel-Prager-Gasse')
	street = street.replace('Baron Karl Gasse','Baron-Karl-Gasse')
	street = street.replace('Bertha V Suttnerstraße','Bertha-von-Suttner-Straße')
	street = street.replace('Bgm. Dr.-Haberl Gasse','Bgm.-Dr.-Haberl-Gasse') # Wiener Neustadt
	street = street.replace('Bischof Faberplatz','Bischof-Faber-Platz')
	street = street.replace('Brunner-Lehensteingasse','Brunner-Lehenstein-Gasse')
	street = street.replace('C Holzmeisterstraße','Clemens-Holzmeister-Straße')
	street = street.replace('Carl Maria Von Weberg ','Carl-Maria-von-Weber-Gasse ')
	street = street.replace('C.V.Hoetzendorfstraße','Conrad-von-Hötzendorf-Straße')
	street = street.replace('Ch. Gudenus Straße','Ch.-Gudenus-Straße') # Hausleiten, eig. Christoph von Gudenus 1688-1800
	street = street.replace('Christoph Gudenus Straße','Ch.-Gudenus-Straße') # auch Christoph-Gudenus-Straße
	street = street.replace('Dr A Lemisch-Straße','Dr.-Arthur-Lemisch-Straße')
	street = street.replace('Dechant Pfeifer Straße','Dechant-Pfeifer-Straße')
	street = street.replace('Dechant Pfeiffer Straße','Dechant-Pfeifer-Straße')
	street = street.replace('Dr.-Eugen Zehme Straße','Dr.-Eugen-Zehme-Straße') # 5111 Bürmoos
	street = street.replace('Dr Fuchs-Gasse','Doktor-Fuchs-Gasse') # 2000 Stockerau
	street = street.replace('Dr.-Habermayergasse','Doktor-Habermayer-Gasse') # 2700 Wiener Neustadt
	street = street.replace('Dr.Ignaz Weber G.','Dr.-Ignaz-Weber-Gasse ') # 2353 Guntramsdorf
	street = street.replace('Dr Karl Rennerstraße','Doktor-Karl-Renner-Straße')
	street = street.replace('Dr.-Karl Schrattenbauerweg','Dr.-Karl-Schrattenbauer-Weg')
	street = street.replace('Dr Kucherweg','Dr.-Kucher-Weg') # 9061 Klagenfurt
	street = street.replace('Dr.-Pascher Straße','Dr.-Pascher-Straße') # 5280 Braunau am Inn
	street = street.replace('Dr.-Robert Griedl Weg','Dr.-Robert-Griedl-Weg') # 8301 Laßnitzhöhe
	street = street.replace('Dr.R.Schuh-Straße','Dr.-Rudolf-Schuh-Straße')
	street = street.replace('Dr.Stumpf Straße','Dr.-Stumpf-Straße')
	street = street.replace('Dr.-Theodor-Körner Straße','Dr.-Theodor-Körner-Straße') # 9523 Landskron
	street = street.replace('Dr.-Viktor Adler Straße','Dr.-Viktor-Adler-Straße') # 2000 Stockerau
	street = street.replace('Eduard-Kittenbergergasse','Eduard-Kittenberger-Gasse')
	street = street.replace('Eduard Pötzl Gasse','Eduard-Pötzl-Gasse')
	street = street.replace('Erich Friedstraße','Erich-Fried-Straße')
	street = street.replace('Erne-Seder Gasse','Erne-Seder-Gasse')
	street = street.replace('Ernst Ludwig Gasse','Ernst-Ludwig-Gasse')
	street = street.replace('Ernst Mach Straße','Ernst-Mach-Straße')
	street = street.replace('Erzherzog Eugen Straße','Erzherzog-Eugen-Straße')
	street = street.replace('Erzherzog Eugenstraße','Erzherzog-Eugen-Straße')
	street = street.replace('Erzherzog Karl Straße','Erzherzog-Karl-Straße')
	street = street.replace('F-Asenbauer-Gasse','Franz-Asenbauer-Gasse')
	street = street.replace('Franz Lehar-Gasse','Franz-Lehar-Gasse')
	street = street.replace('Fritz Preglgasse','Fritz-Pregl-Gasse')
	street = street.replace('Franz Asenbauergasse','Franz-Asenbauer-Gasse')
	street = street.replace('Franz Asenbauer-Gasse','Franz-Asenbauer-Gasse')
	street = street.replace('Franz Blasser-Straße','Franz-Plasser-Straße') # 5280 Braunau am Inn
	street = street.replace('Franz Fischerstraße','Franz-Fischer-Straße') # 6020 Innsbruck
	street = street.replace('Franz-Jaenkl-Straße','Franz-Jänkl-Straße')
	street = street.replace('Frau Hitt Straße','Frau-Hitt-Straße')
	street = street.replace('F.J. Weizeneggerweg','Franz-Josef-Weizenegger-Weg')
	street = street.replace('Fred Zinnemannplatz','Franz-Zinnemann-Platz')
	street = street.replace('Fridtjof-Nansenstraße','Fridtjof-Nansen-Straße') # 9800 Spittal an der Drau
	street = street.replace('Friedrich Attlmayr Straße','Friedrich-Attlmayr-Straße') # 6600 Reutte
	street = street.replace('Friedrich Hegel-Gasse','Friedrich-Hegel-Gasse') # 2232 Deutsch-Wagram
	street = street.replace('Friedr.Kaiser-Gasse','Friedrich-Kaiser-Gasse')
	street = street.replace('Friedrich Schiller Straße','Friedrich-Schiller-Straße') # 2340 Mödling
	street = street.replace('Friedrich Schmidt Platz','Friedrich-Schmidt-Platz')
	street = street.replace('Friedrich Teller Gasse','Friedrich-Teller-Gasse')
	street = street.replace('Georg Bilgeri Straße','Georg-Bilgeri-Straße')
	street = street.replace('Georg Lora Straße','Georg-Lora-Straße')
	street = street.replace('G.-Grinninger-Straße','Georg-Grinninger-Straße')
	street = street.replace('Gerhart Ellertstraße','Gerhart-Ellert-Straße') # 9400 Wolfsberg
	street = street.replace('Hans Untermüller Straße','Hans-Untermüller-Straße') # 6020 Innsbruck
	street = street.replace('H.V.Hofmannst.Weg','Hugo-von-Hofmannsthal-Weg')
	street = street.replace('Heiligenstaedter Straße','Heiligenstädter Straße')
	street = street.replace('Heinrich Leflergasse','Heinrich-Lefler-Gasse')
	street = street.replace('Herzmanovsky Orlandogasse','Herzmanovsky-Orlando-Gasse')
	street = street.replace('Ing Julius Raab-Straße','Ing.-Julius-Raab-Straße')
	street = street.replace('Ingenuin-Fischlerstraße','Ingenuin-Fischler-Straße')
	street = street.replace('Jakob Sereiniggstraße','Jakob-Sereinigg-Straße')
	street = street.replace('J. Fux Straße','Johann-Fux-Straße')
	street = street.replace('J.Haagnstraße','Julius-Haagn-Straße')
	street = street.replace('Johann Böhmstraße','Johann-Böhm-Straße')
	street = street.replace('Johann Neumayer Gasse','Johann-Neumayer-Gasse')
	street = street.replace('Johann Weber Straße','Johann-Weber-Straße')
	street = street.replace('J.W.Kleinstraße','Johann-Wilhelm-Klein-Straße')
	street = street.replace('Johannes Gutenberg Straße','Johannes-Gutenberg-Straße') # Wiener Neustadt
	street = street.replace('Josef-Prechtlweg','Josef-Prechtl-Weg') # Braunau am Inn
	street = street.replace('Josef Schwarz Straße','Josef-Schwarz-Straße')
	street = street.replace('Josef Sirowy-Straße','Josef-Sirowy-Straße')
	street = street.replace('Kaiserschuetzenstraße','Kaiserschützenstraße')
	street = street.replace('K.Billerhartstraße','Karl-Billerhart-Straße')
	street = street.replace('Karl Schwarz Straße','Karl-Schwarz-Straße')
	street = street.replace('Karl Steiger-Straße','Karl-Steiger-Straße')
	street = street.replace('Konr.Hopferwieserg.','Konrad-Hopferwieser-Gasse ')
	street = street.replace('Kuno-Brandauerstraße','Kuno-Brandauer-Straße')
	street = street.replace('Laaer Berg Straße','Laaer-Berg-Straße')
	street = street.replace('Laaerbergstraße','Laaer-Berg-Straße')
	street = street.replace('Laaerberg Straße','Laaer-Berg-Straße')
	if (city == 'Sollenau'):
		street = street.replace('Gartst.M.The.Laercheng','Lärchengasse 5') # qrz.com
		city = 'Gartenstadt Maria Theresia'
	street = street.replace('Leonard Bernstein Straße','Leonard-Bernstein-Straße')
	street = street.replace('L. Kunschakgasse','Leopold-Kunschak-Gasse') # 2225 Zistersdorf
	street = street.replace('Ludo Hartmanngasse','Ludo-Hartmann-Gasse')
	street = street.replace('Ludwig Laab Straße','Ludwig-Laab-Straße') # 2000 Stockerau
	street = street.replace('Margaretengtl','Margaretengürtel')
	street = street.replace('Mautner-Markhofgasse','Mautner-Markhof-Gasse')
	street = street.replace('Maurer Lange-Gasse','Maurer Lange Gasse')
	street = street.replace('Mayr-Melnhofstraße','Mayr-Melnhof-Straße') # 8700 Leoben
	if (city == 'Bregenz'):
		street = street.replace('Michael Gaismeierstraße','Michael-Gaismayr-Straße') # typo # Bregenz, in Innsbruck Gaismair
	street = street.replace('Michl Felder Straße','Michl-Felder-Straße')
	street = street.replace('Michl Felderstraße','Michl-Felder-Straße')
	street = street.replace('M Unterlercher-Straße','Michael-Unterlercher-Straße')
	street = street.replace('M Wachberger Straße','Michael-Wachberger-Straße')
	if (city == 'Kapfenberg'):
		street = street.replace('Mühlbacherstraße','Anton-Mühlbacher-Straße')
	street = street.replace('Nico Dostal Straße','Nico-Dostal-Straße')
	if (city == 'Braunau am Inn'):
		street = street.replace('Pascherstraße','Dr.-Pascher-Straße')
	street = street.replace('Per Albin Hanssonstraße','Per-Albin-Hansson-Straße')
	street = street.replace('Peter Altenberg-Gasse','Peter-Altenberg-Gasse')
	street = street.replace('Peter Anich Straße','Peter-Anich-Straße') # 6300 Wörgl
	street = street.replace('Peter Henleinweg','Peter-Henlein-Weg')
	street = street.replace('Peter Jordanstraße','Peter-Jordan-Straße')
	street = street.replace('Peter Pfenningerstraße','Peter-Pfenninger-Straße')
	street = street.replace('Pfarrer Matz Gasse','Pfarrer-Matz-Gasse')
	street = street.replace('Prinz Eugen Straße','Prinz-Eugen-Straße')
	street = street.replace('Prinz Eugenstraße','Prinz-Eugen-Straße')
	street = street.replace('Raaberbahngasse','Raaber-Bahn-Gasse')
	street = street.replace('Richard Straussgasse','Richard-Strauss-Gasse')
	street = street.replace('Richard Wagnergasse','Richard-Wagner-Gasse')
	street = street.replace('Robert Ahlfeld Straße','Robert-Ahlfeld-Straße') # 2000 Stockerau
	street = street.replace('Robert-Häuserstraße','Robert-Häuser-Straße') # 2544 Leobersdorf/Siebenhaus
	street = street.replace('Robert Kochstraße','Robert-Koch-Straße') # 4600 Wels
	street = street.replace('Rudolf Waisenhorn-Gasse','Rudolf-Waisenhorn-Gasse')
	street = street.replace('Schuettelstraße','Schüttelstraße')
	street = street.replace('Sebastian Bauer Straße','Sebastian-Bauer-Straße')
	street = street.replace('St. Bartholomäusplatz','St.-Bartholomäus-Platz')
	street = street.replace('St.Gerold Weg','Sankt-Gerold-Weg') # 6830 Brederis
	street = street.replace('Theodor Koernerstraße','Theodor-Körner-Straße') # 8010 Graz
	street = street.replace('Theodor Storm Straße','Theodor-Storm-Straße') # 8041,8042 Graz
	street = street.replace('TH. Pampichler-Straße','Theresia-Pampichler-Straße') # 2000 Stockerau
	street = street.replace('Thomas Edisonstraße','Thomas-Edison-Straße')
	street = street.replace('Thomas Morusgasse','Thomas-Morus-Gasse')
	street = street.replace('Tuerkenstraße','Türkenstraße')
	street = street.replace('Veichtlbauerweg','Johann-Veichtlbauer-Weg') # 4910 Ried im Innkreis
	street = street.replace('Viktor Dankl Straße','Viktor-Dankl-Straße') # 6020 Innsbruck
	street = street.replace('Weissgerberlände','Weißgerberlände')
	street = street.replace('Wilhelm Kressplatz','Wilhelm-Kreß-Platz')
	if (city == 'Deutsch-Wagram'):
		street = street.replace('Wilhelm Kressgasse','Wilhelm-Kress-Gasse') # Deutsch-Wagram
	else:
		street = street.replace('Wilhelm Kressgasse','Wilhelm-Kreß-Gasse') # Wien 22, Guntramsdorf,  Pressbaum
# Streets with location names
	street = street.replace('Alser-Straße','Alser Straße')
	street = street.replace('Altenbergerstraße','Altenberger Straße')
	street = street.replace('Altmannsdorferstraße','Altmannsdorfer Straße')
	street = street.replace('Atzgersdorferstraße','Atzgersdorfer Straße')
	street = street.replace('Breitenfurterstraße','Breitenfurter Straße')
	street = street.replace('Breitenleerstraße','Breitenleer Straße')
	street = street.replace('Breitenseerstraße','Breitenseer Straße')
	street = street.replace('Dresdnerstraße','Dresdner Straße')
	street = street.replace('Freistaedterstraße','Freistädter Straße')
	street = street.replace('Freistädterstraße','Freistädter Straße')
	street = street.replace('Gaalerstraße','Gaaler Straße') # 8720 Knittelfeld
	street = street.replace('Gerasdorferstraße','Gerasdorfer Straße')
	street = street.replace('Groedigerstraße ','Grödiger Straße ')
	street = street.replace('Grödigerstraße','Grödiger Straße')
	street = street.replace('Gumpendorferstraße','Gumpendorfer Straße')
	street = street.replace('Harmannsdorferstraße','Harmannsdorfer Straße')
	street = street.replace('Heiligenstädterstraße','Heiligenstädter Straße')
	street = street.replace('Hetzendorferstraße','Hetzendorfer Straße')
	street = street.replace('Hörtendorferstraße','Hörtendorfer Straße')
	street = street.replace('Italienerstraße','Italiener Straße')
	street = street.replace('Jedleseerstraße','Jedleseer Straße')
	street = street.replace('Josefstädterstraße','Josefstädter Straße')
	street = street.replace('Kahlenbergerstraße','Kahlenberger Straße')
	street = street.replace('Kagranerplatz','Kagraner Platz')
	street = street.replace('Kaiserebersdorferstraße','Kaiserebersdorfer Straße')
	street = street.replace('Kirchbergerstraße','Kirchberger Straße')
	street = street.replace('Königstetterstraße','Königstetter Straße')
	street = street.replace('Korneuburgerstraße','Korneuburger Straße') # 2103 Langenzersdorf
	street = street.replace('Krottendorferstraße','Krottendorfer Straße') # 2103 Langenzersdorf
	street = street.replace('Langenzersdorferstraße','Langenzersdorfer Straße')
	street = street.replace('Leobendorferstraße','Leobendorfer Straße') # 2100 Unterrohrbach
	street = street.replace('Lerchenfelderstraße','Lerchenfelder Straße')
	street = street.replace('Liesingerstraße','Liesinger Straße') # 3012 Wolfsgraben
	street = street.replace('Mattighofenerstraße','Mattighofner Straße') # typo ! Braunau am Inn
	street = street.replace('Millstätterstraße','Millstätter Straße') #9523 Landskron (Villach)
	street = street.replace('Neudorferstraße','Neudorfer Straße')
	street = street.replace('Nußdorferstraße','Nußdorfer Straße')
	street = street.replace('Ottakringerstraße','Ottakringer Straße')
	street = street.replace('Penzingerstraße','Penzinger Straße')
	street = street.replace('Pontlatzerstraße','Pontlatzer Straße')
	street = street.replace('Schillingsdorferstraße','Schillingsdorfer Straße')
	street = street.replace('Schönbrunnerstraße','Schönbrunner Straße')
	street = street.replace('Schweizerstraße','Schweizer Straße') # 2020 Hollabrunn
	street = street.replace('Sechshauserstraße','Sechshauser Straße')
	street = street.replace('Speisingerstraße','Speisinger Straße')
	street = street.replace('Stammersdorferstraße','Stammersdorfer Straße')
	street = street.replace('Stockerauerstraße','Stockerauer Straße') # 2100
	street = street.replace('Strassganger Straße','Straßganger Straße')
	street = street.replace('St. Veiterstraße','St. Veiter Straße')
	street = street.replace('St.-Veiter-Straße','St. Veiter Straße')
	street = street.replace('Traundorferstraße','Traundorfer Straße')
	street = street.replace('Vösendorferstraße','Vösendorfer Straße')
	street = street.replace('Währingerstraße','Währinger Straße')
	street = street.replace('Waidhofnerstraße','Waidhofner Straße')
	street = street.replace('Weimarerstraße','Weimarer Straße')
	street = street.replace('Wienerstraße','Wiener Straße')
	street = street.replace('Wr Neustädterstraße','Wiener Neustädter Straße')
	return street,city
def corr_city(city,postal_code):
	city = city.replace('Aigen Im Mühlkreis','Aigen-Schlägl') # Gemeinde Aigen im Mühlkreis und Schlägl 2015 zusammengelegt
	city = city.replace('Aigen/Muehlkr.','Aigen-Schlägl')
	#city = city.replace('Alberndorf In Der Riedmark','Alberndorf in der Riedmark')
	city = city.replace(' In Der ',' in der ')
	#city = city.replace('Allerheiligen Bei Wildon','Allerheiligen bei Wildon')
	city = city.replace(' Bei ',' bei ')
	#city = city.replace('Altenberg Bei Linz','Altenberg bei Linz')
	#city = city.replace('Altenmarkt An Der Triesting','Altenmarkt an der Triesting')
	city = city.replace(' An Der ',' an der ')
	#city = city.replace('Altenmarkt Im Pongau','Altenmarkt im Pongau')
	city = city.replace(' Im ',' im ')
	# city = city.replace('Altenmarkt','Altenmarkt an der Triesting') # does not work, use:
	if (city == 'Altenmarkt'): city = 'Altenmarkt an der Triesting'
	city = city.replace('Altmuenster','Altmünster')
	#city = city.replace('Annaberg Im Lammertal','Annaberg im Lammertal')
	#city = city.replace('Arzl Im Pitztal','Arzl im Pitztal')
	#city = city.replace('Aschach An Der Donau','Aschach an der Donau')
	city = city.replace('Aschau/,Zillertal','Aschau')
	#city = city.replace('Bad Goisern Am Hallstättersee','Bad Goisern am Hallstättersee')
	city = city.replace(' Am ',' am ')
	if (city == 'Bad Goisern'): city = 'Bad Goisern am Hallstättersee'
	#city = city.replace('Bad Sankt Leonhard Im Lavanttal','Bad Sankt Leonhard im Lavanttal')
	#city = city.replace('Breitenfurt Bei Wien','Breitenfurt bei Wien')
	if (city == 'Breitenfurt'): city = 'Breitenfurt bei Wien'
	#city = city.replace('Bruck An Der Leitha','Bruck an der Leitha')
	city = city.replace(' An Der ',' an der ')
	#city = city.replace('Bruck An Der Mur','Bruck an der Mur')
	#city = city.replace('Bruck Am Ziller','Bruck am Ziller')
	#city = city.replace('Brunn Am Gebirge','Brunn am Gebirge')
	#city = city.replace('Brunn An Der Wild','Brunn an der Wild')
	#city = city.replace('Bubendorf Im Burgenland','Bubendorf im Burgenland')
	city = city.replace(' Im ',' im ')
	if (city == 'Dellach'): city = 'Dellach im Drautal'
	if (city =='Fliess'): city = 'Fließ'
	#city = city.replace('Hadersdorf Am Kamp','Hadersdorf am Kamp')
	#city = city.replace('Hall In Tirol','Hall in Tirol')
	city = city.replace(' In ',' in ')
	#city = city.replace('Laa An Der Thaya','Laa an der Thaya')
	if (city == 'Rohrbach'): city = 'Rohrbach-Berg'
	#city = city.replace('Sankt Jakob Im Rosental','Sankt Jakob im Rosental')
	#city = city.replace('Sankt Marein Bei Graz-Markt','Sankt Marein bei Graz')
	#city = city.replace('Sankt Veit An Der Glan','Sankt Veit an der Glan')
	#city = city.replace('Schwarzau Am Steinfeld','Schwarzau am Steinfeld')
	#city = city.replace('Steindorf Am Ossiacher See','Steindorf am Ossiacher See')
	#city = city.replace('Steinhaus Am Semmering','Steinhaus am Semmering')
	if (city == 'Uttendorf' and postal_code == '5261'):
		city = 'Helpfau-Uttendorf'
# TYPO
	city = city.replace('Breittenfurt','Breitenfurt')
	city = city.replace('Bsichofshofen','Bischofshofen')
	city = city.replace('Maria-Anzbach','Maria Anzbach') # Namensänderung der Marktgemeinde 2022 Landesgesetzblatt Nr. 44/2022
	city = city.replace('Sankt Andrä/Wördern','Sankt Andrä-Wördern')
	city = city.replace('St.Andrä-Wördern','Sankt Andrä-Wördern')
	city = city.replace('Sankt Andrä Wördern','Sankt Andrä-Wördern')
	city = city.replace('Waidhofen/Ybbs','Waidhofen an der Ybbs')
	#city = city.replace('Wr.Neustadt','Wiener Neustadt')
	city = city.replace('Wr Neustadt','Wiener Neustadt')
	city = city.replace('Wörther See','Wörthersee')
# Postleitzahl Änderungen
	if (postal_code == '8071' and city == 'Gössendorf'): postal_code = '8077'
	if (postal_code == '6064'): postal_code = '6063' # Postamt Rum Dorf aufgelassen, nur mehr 6063
	if (postal_code == '6020' and city == 'Rum'): postal_code = '6063' # Serlesstraße in Rum
	return city, postal_code
def call_postal_code(location, address, verbose):
	street = address
	if location == '' or location[0] == '*':
		return '', '', street
	p = location.split(' ',1)
	if isinteger(p[0]) and int(p[0]) > 1000 and int(p[0]) < 10000:
		postal_code = p[0]
		city, postal_code = corr_city(p[1].strip(),postal_code)
		street,city = corr_street(address,city)
	else:
		postal_code, city, street = call_change_city(location, address, verbose)

	# street = corr_street(street)
	# add city and street to the database
	return postal_code, city, street

def call_data_record(line, mod_date, verbose, cur):

	# we have to split the record with a cost-intensive regular expression
	# record = re.split('OE[0-9][A-Z]{1,3}[ \t]{3,20}',line) # this does not work 100%
	# record = re.findall(r'(OE[0-9][A-Z]{1,3})[ \t]{2,12}([A-ZÄÖÜ].+[ ]?.*[ ]?.*[ ]?.*)[ \t]{3,30}(.{3,30})[ \t]{3,30}([1,3,4])', line)
	# record = re.search(r'(OE[0-9][A-Z]{1,3})[ ]{2,12}([. ]+)[ ]{3,50}([. ]+)[ ]{3,50}([1-4])', line)
	# record = re.search(r'^(OE[0-9][A-Z]{1,3})[ \t]{2,20}([\w ]{1,12})[ ]{3,50}(.*)([1-4]{1})$', line)

	# Never split Addresses containing 2 or 3 spaces, also several records contain no address or no location
	records = re.split(r'[ ]{4,65}', line)
	# [records for record in records]

	if verbose > 2 :
		print(f'Record length: {len(records)}')

		for m in records:
			print(m)

	# HARDENING:
	assert(len(records) == 5)
	# OE Call:
	call = records[0]
	call = fix_call(call,verbose)
	match = re.search(r'^(OE[0-9][A-Z]{1,3})', call)
	assert(match.string == call)
	fullname = records[1]
	location = records[2]
	address  = records[3]
	permit_class = records[4]
	fullname = fix_typo(call, fullname, verbose)
	firstname = ''
	surname = ''
	# If there is a clubstation
	if is_clubstation(call):
		# Name starting with only one quotation marks e.g. " -- remove that one:
		fullname = remove_first_quote_if_odd(fullname, verbose) # only found @ clubstations
		clubstationfile = '.callbook_club'
		if verbose > 1:
			print(f'Call: {call}, Name: {fullname}')
		path = os.path.join(os.path.dirname(os.path.abspath(__file__)), clubstationfile)
		if os.path.exists(path):
			fullname = replace_substring_with_line(path, fullname, verbose)
		gender = '*'
		firstname = fullname.strip()
	elif fullname[0] == '*':
		gender = '*'
	else: # Try to split the YL or OMs Name, guess the gender
		firstname, surname, gender = call_split_name(fullname, call, verbose)
	if verbose > 1:
		if gender == '*':
			print(f'Call: {call}, Name: {fullname}, Gender: {gender}')
		else:
			print(f'Call: {call}, First Name: {firstname}, Surname: {surname}, Gender: {gender}')
		print(f'Location: {location}, Address: {address}, Permit: {permit_class}')

	created = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
	call_data_record.cnt += 1 # increment the User_id
	user_id = call_data_record.cnt

	statement = f'SELECT `call`, `firstname`, `surname`, `gender` FROM `callbook_user` WHERE `call` = "{call}" AND `active` = 1'
	cur.execute(statement)
	result = cur.fetchall()
	if not result:
		postal_code,city,street = call_postal_code(location, address, verbose)
		statement = "INSERT INTO `callbook_address`(`location`,`address`,`postal_code`,`city`,`street`) VALUES (%s,%s,%s,%s,%s)"
		data = (location,address,postal_code,city,street)
		try: cur.execute(statement, data)
		except mariadb.Error as e:
			print(f'\n[WARN] MySQLError during execute statement\n\tArgs: {e.args}')
		except Exception as e:
			print('Error: {}'.format(e), file=sys.stderr)

		statement = "INSERT INTO `callbook_user`(`user_id`,`call`,`firstname`,`surname`,`gender`,`created`,`modified`,`address_id`) VALUES (%s, %s, %s, %s, %s, %s, %s, LAST_INSERT_ID())"
		data = (user_id,call,firstname,surname,gender,created,created)
		try:
			cur.execute(statement, data)
		except mariadb.Error as e:
			print(f'\n[WARN] MySQLError during execute statement\n\tArgs: {e.args}')
		except Exception as e:
			print('Error: {}'.format(e), file=sys.stderr)
	else:
		for r in result:
			print(f'Call: {call} {firstname} {surname} {gender} <- ')

call_data_record.cnt = 0

def call_analyse_pdf(file, verbose, cur):

	# Define a regular expression to match tables

	reader = PdfReader(file)
	meta = reader.metadata
	if verbose:
		print(verbose)
		print('   Pages:', len(reader.pages))
		# All of the following could be None!
		print(f'  Author: {meta.author}')
		print(f' Creator: {meta.creator}')
		print(f'Producer: {meta.producer}')
		print(f' Subject: {meta.subject}')
		print(f'   Title: {meta.title}')
		print(f' Created: {meta.creation_date}')
		print(f'Modified: {meta.modification_date}')

	for page in reader.pages:
		page_text = page.extract_text(extraction_mode="layout", layout_mode_space_vertically=False)
		lines = page_text.strip().splitlines()
		for line in lines[3:-2]:
			line = line.strip()

			if verbose >= 2:
				print(line)
			call_data_record(line, meta.modification_date,verbose, cur)

def exec_sql_file(cursor, sql_file):
	statement = ''
	try:
		for line in open(sql_file):
			if line.strip().startswith('--'):  # ignore sql comment lines
				continue
			if line.strip().endswith(';'):  # keep appending lines that don't end in ';'
				statement += line
				try:
					cursor.execute(statement)
				except mariadb.Error as e: # (OperationalError, ProgrammingError) as e:
					print(f'\n[WARN] MySQLError during execute statement\n\tArgs: {e.args}')
				statement = ''
			else:
				statement += line

	except FileNotFoundError:
		print(f'The file {path} was not found.')
	except Exception as e:
		print('Error: {}'.format(e), file=sys.stderr)

def call_create_database(configdb):
	database = configdb.get('database','callbook')
	print(f'Try to create the database "{database}" (if you have enough privileges)')
	conn = mariadb.connect(
		host = configdb.get('host','127.0.0.1'),
		user = configdb.get('user','om'),
		password = configdb.get('password','oe3tkt')
		)
	# create a cursor object
	cur = conn.cursor()
	# creating database
	try:
		cur.execute('CREATE DATABASE ' + database)
	except mariadb.Error as e:
		print(f'\n[ERROR] MySQLError during execute statement\n\tArgs: {e.args}')
		sys.exit(5)
	cur.execute('SHOW DATABASES')
	databaseList = cur.fetchall()
	#for database in databaseList:
	#	print(database)
	if not databaseList:
		conn.close()
		sys.exit(5)
	cur.execute('USE ' + database)

	return conn

if __name__ == '__main__':
	args = call_parser()
	config = call_configparser()
	try:
		filename = call_website(**vars(args))
		if args.verbose > 1:
			print(f'Filename: {filename}')

	except Exception as e:
		print('Error: {}'.format(e), file=sys.stderr)
		sys.exit(1)

	try:
		configdb = config['MariaDB'] # fetch from config file, then do with fallbacks:
		conn = mariadb.connect(
			user = configdb.get('user', 'om'),
			password = configdb.get('password','oe3tkt'),
			host = configdb.get('host','127.0.0.1'),
			port = configdb.getint('port',3306),
			database = configdb.get('database','callbook')
		)
	except mariadb.Error as e:
		print(f'Error connectiong to MariaDB platform: {e}')
		conn = call_create_database(configdb)

	print(f'Connected to MariaDB API Version {mariadb.mariadbapi_version}.')

	print(datetime.datetime.now(datetime.UTC))
	# Get Cursor
	cur = conn.cursor()
	cur.execute('SELECT VERSION()')
	print(cur.fetchone()[0])
	sql_file = '.sql_init'
	path = os.path.join(os.path.dirname(os.path.abspath(__file__)), sql_file)
	exec_sql_file(cur, path)

	call_analyse_pdf(filename,args.verbose,cur)
	conn.commit()
	cur.close()
	conn.close()
	sys.exit(0)
